ParallelIterator

Struct ParallelIterator 

pub struct ParallelIterator<O> { /* private fields */ }

Implementations

Source

impl<O> ParallelIterator<O>

pub fn new<PC, XC, P, X, I>(producer_ctor: PC, xform_ctor: XC) -> Self
where PC: 'static + Send + FnOnce() -> P, XC: 'static + Send + Sync + Fn() -> X, X: FnMut(I) -> O, I: 'static + Send, O: 'static + Send, P: IntoIterator<Item = I>,

  • PC - Producer Constructor. Enables usage of !Send and !Sync objects in the producer function.

  • XC - Xform (closure) Constructor. Enables usage of !Send and !Sync objects in the transform closure. This can be used for creating thread local caches, like large allocations re-used by different tasks, packaged as a closure.

  • P - Producer iterator. Consumed internally by the transform/worker threads.

  • X - Xform closure. Applied to each job item produced by the producer iterator, in parallel by multiple worker threads. This can be FnMut since it’s owned by a dedicated worker thread and will never be called by some other thread. The closure can safely store and reuse mutable resources between job items, for example large memory allocations.

  • I - Input item. Or task, produced by the producer iterator, transformed by the Xform closures.

  • O - Output item. Returned by the Xform closure(s) and by the Iterator::next method.

Examples found in repository?
examples/example_1.rs (line 10)
9fn main() {
10    for i in ParallelIterator::new(|| (0u32..100), || do_some_work) {
11        println!("Got a number: {}!", i);
12    }
13}
More examples
Hide additional examples
examples/example_2.rs (line 21)
11fn main() {
12    const MAX: usize = 1000;
13    let xform_ctor = || {
14        let mut buffer = Vec::with_capacity(MAX);
15        move |i| {
16            buffer.clear(); // Clear but keep the internal allocation.
17            do_some_work(i, &mut buffer);
18            buffer.last().map(|u| *u) // This is just an example value.
19        }
20    };
21    for i in ParallelIterator::new(|| (0..MAX), xform_ctor) {
22        match i {
23            Some(i) => println!("Got Some({})!", i),
24            None => println!("Got None!"),
25        }
26    }
27}